import java.io.*;


class LogOut
{
    private static final String  OLD_EXT	   = ".old";
    private static final int	 MAX_NR_FILE_LINES = 5000;

    private String	 file;
    private PrintWriter  file_out;
    private int		 file_line_nr;


    LogOut (Param  param)
    {
	if (param.useLogFile ())
	{
	    file = param.getLogFile ();
	    openFile (true);
	}
	else
	{
	    file     = null;
	    file_out = null;
	}
    }


    void  printLine (String  line,
		     int     id)
    {
	if (file_out == null)
	{
	    System.out.println (getIDNrThreadsLine (line, id));
	    System.out.flush   ();
	}
	else
	{
	    synchronized (this)
	    {
		file_out.println (getIDNrThreadsLine (line, id));
		file_out.flush	 ();

		if (++file_line_nr >= MAX_NR_FILE_LINES)
		    renameReopenFile (id);
	    }
	}
    }


    void  printLines (String  lines,
		      int     nr_lines,
		      int     id)
    {
	if (file_out == null)
	{
	    System.out.print (lines);
	    System.out.flush ();
	}
	else
	{
	    synchronized (this)
	    {
		file_out.print (lines);
		file_out.flush ();

		if ( (file_line_nr += nr_lines) >= MAX_NR_FILE_LINES )
		    renameReopenFile (id);
	    }
	}
    }


    private void  renameReopenFile (int  id)
    {
	try
	{
	    String  old_file = file + OLD_EXT;;
	    
	    File  file_object	  = new File (file    );
	    File  old_file_object = new File (old_file);

	    if (old_file_object.isFile ())
		old_file_object.delete ();

	    file_object.renameTo (old_file_object);
	}
	catch (SecurityException  secure_except)
	{
	    printLine (secure_except.getMessage (), id);
	}
	finally
	{
	    openFile (false);
	}
    }


    private void  openFile (boolean  check_append)
    {
	try
	{
	    boolean  append;

	    if (check_append  &&  (file_line_nr = getNrLogLines ()) > 0)
	    {
		append = true;
	    }
	    else
	    {
		file_line_nr = 0;
		append	 = false;
	    }

	    file_out = new PrintWriter (
			       new OutputStreamWriter (
				       new FileOutputStream (file,
							     append)),
			       true);
	}
	catch (Exception  except)
	{
	    System.err.println (Param.JAVA_NAME + ": " +
				except.getMessage () +
				": Check & open log file: " + file);
	    System.exit        (1);
	}
    }


    private int  getNrLogLines ()  throws  SecurityException,
					   IOException
    {
	File  file_object = new File (file);
	int   line_nr	  = 0;

	if ( !file_object.isFile () )
	    return line_nr;

	BufferedReader  file_in;
	
	file_in = new BufferedReader (
			  new InputStreamReader (
				  new FileInputStream (file)));

	String  line;

	while ( (line = file_in.readLine ()) != null )
	    line_nr++;

	file_in.close ();

	return line_nr;
    }


    private static String  getIDNrThreadsLine (String  line,
					       int     id)
    {
	StringBuffer  id_line_sb = new StringBuffer   ("[");
	int	      nr_threads = Thread.activeCount (   );

	id_line_sb.append (id	     );
	id_line_sb.append (",#"	     );
	id_line_sb.append (nr_threads);
	id_line_sb.append ("]  "     );
	id_line_sb.append (line      );

	return id_line_sb.toString ();
    }
}
