import java.io.*;
import java.util.*;


class  Param
{
    private static final String DFT_PROGRAM_DIR = ".";
    private static final String DFT_PROGRAM	= "net_test_com_server";

    private static final String []  BASE_COMMANDS_ALLOWED =
    {
	"start_netserver",
	"start_udpmon_resp",
	"host_net_test"
    };

    static final String  JAVA_NAME   = "java NetTestComServer";
    static final String  STDIN_FILE  = "-";
    static final String  COMMENT_STR = "#";
    static final String  EQUIV_STR   = "=";
    static final String  HOME_VAR    = "$HOME";
    static final String  PROGRAM_VAR = "$PROGRAM";
    static final int     DFT_PORT    = 37373;

    private SysProp    sys_prop;
    private String     program_dir;
    private String     program;
    private String     home;
    private boolean    debug;
    private String     log_file;
    private Hashtable  host_allow_hash;
    private Hashtable  base_com_allow_hash;
    private boolean    accept_all;
    private int	       port;


    Param (SysProp  sys_prop)
    {
	this.sys_prop = sys_prop;

	initialise ();
    }


    private void  initialise ()
    {
	program_dir	= DFT_PROGRAM_DIR;
	program		= DFT_PROGRAM;
	home		= null;
	debug		= false;
	log_file	= STDIN_FILE;
	accept_all	= true;
	host_allow_hash = new Hashtable ();
	port		= DFT_PORT;

	setBaseComAllowHash ();
    }


    private void  setBaseComAllowHash ()
    {
	base_com_allow_hash = new Hashtable ();

	int  nr_base_coms_allow = BASE_COMMANDS_ALLOWED.length;

	for (int index = 0;  index < nr_base_coms_allow;  index++)
	{
	    String  base_com = BASE_COMMANDS_ALLOWED [index];

	    base_com_allow_hash.put (base_com, base_com);
	}
    }


    void  parseArguments (String []  argv)
    {
	int argv_length = argv.length;
	int  index	= 0;

	while (index < argv_length)
	{
	    String  arg = argv [index++];

	    if (arg.equals ("-debug"))
	    {
		debug = true;
	    }
	    else if (arg.equals ("-config_file"))
	    {
		checkOptionIndex (arg, index, argv_length);
		parseConfigFile	 (argv [index++]);
	    }
	    else if (arg.equals ("-home"))
	    {
		checkOptionIndex (arg, index, argv_length);
		home = argv [index++];
	    }
	    else if (arg.equals ("-program_dir"))
	    {
		checkOptionIndex (arg, index, argv_length);
		program_dir = argv [index++];
	    }
	    else if (arg.equals ("-program"))
	    {
		checkOptionIndex (arg, index, argv_length);
		program = argv [index++];
	    }
	    else if (arg.equals ("-log_file"))
	    {
		checkOptionIndex (arg, index, argv_length);
		
		log_file = getParseFileName (argv [index++]);
	    }
	    else if (arg.equals ("-pid_file"))
	    {
		checkOptionIndex (arg, index, argv_length);
		index++;
	    }
	    else if (arg.equals ("-allow_host"))
	    {
		checkOptionIndex (arg, index, argv_length);

		String host = argv [index++];

		host_allow_hash.put (host, host);
		accept_all = false;
	    }
	    else if (arg.equals ("-port"))
	    {
		checkOptionIndex (arg, index, argv_length);

		port = getCheckInt (argv [index++]);
	    }
	    else
	    {
		printUsage  ( );
		System.exit (1);
	    }
	}
    }


    private void  parseConfigFile (String  config_file)
    {
	config_file = getParseFileName (config_file);

	try
	{
	    BufferedReader  config_in;

	    config_in = new BufferedReader (
				new InputStreamReader (
					new FileInputStream (
						config_file)));

	    StringBuffer  config_expr_sb = null;
	    int		  line_nr	 = 0;
	    String	  line;

	    while ( (line = config_in.readLine ()) != null )
	    {
		line_nr++;

		int  comment_index = line.indexOf (COMMENT_STR);

		if (comment_index >= 0)
		    line = line.substring (0, comment_index);

		line = line.trim ();

		int  equiv_index = line.indexOf (EQUIV_STR);

		if (equiv_index >= 0)
		{
		    if (config_expr_sb != null)
			parseConfigExpr (config_expr_sb, line_nr - 1);

		    config_expr_sb = new StringBuffer (line);
		}
		else if (line.length () > 0)
		{
		    if (config_expr_sb != null)
		    {
			config_expr_sb.append (" " );
			config_expr_sb.append (line);
		    }
		    else
		    {
			config_expr_sb = new StringBuffer (line);
		    }
		}
	    }

	    if (config_expr_sb != null)
		parseConfigExpr (config_expr_sb, line_nr);

	    config_in.close ();
	}
	catch (IOException  io_except)
	{
	    System.err.println (JAVA_NAME + ": " +
				io_except.getMessage () +
				": Config file: " +
				config_file);
	    System.exit        (1);
	}
    }


    private void  parseConfigExpr ( StringBuffer  config_expr_sb,
				    int		  line_nr )
    {
	String  config_expr = config_expr_sb.toString ();

	int equiv_index = config_expr.indexOf (EQUIV_STR);

	if ( equiv_index > 0  &&
	     equiv_index < config_expr.length () - 1 )
	{
	    String  name  = config_expr.substring (0, equiv_index);
	    String  value = config_expr.substring (equiv_index + 1);

	    name  = name .trim ();
	    value = value.trim ();

	    if (name.length () > 0  &&  value.length () > 0)
	    {
		parseConfigArguments (name, value);

		return;
	    }
	}

	System.err.println (JAVA_NAME +
			    ": Bad configuration expression at line #" +
			    line_nr + ": " + config_expr);
	System.exit	    (1);
    }


    private void  parseConfigArguments ( String  name,
					 String  value )
    {
	
	StringTokenizer  value_st = new StringTokenizer (value);

	int argv_length = 1 + value_st.countTokens ();

	String []  config_argv = new String [argv_length];

	config_argv [0] = "-" + name;

	for (int index = 1;  index < argv_length;  index++)
	    config_argv [index] = value_st.nextToken ();

	parseArguments (config_argv);
    }


    private String  getParseFileName (String  file_name)
    {
	if (home == null)
	    home = sys_prop.getHome ();

	file_name = getSubstitute (file_name, HOME_VAR,    home);
	file_name = getSubstitute (file_name, PROGRAM_VAR, program);

	return file_name;
    }


    private static String  getSubstitute ( String  subs,
					   String  src,
					   String  dest )
    {
	int  dest_length = dest.length ();
	int  subs_ind	 = 0;
	int  src_ind;

	while ( (src_ind = subs.indexOf (src, subs_ind)) >= 0 )
	{
	    String  begin = subs.substring (0, src_ind);
	    String  end	  = subs.substring (src_ind + src.length ());

	    subs     = begin + dest + end;
	    subs_ind = src_ind + dest_length;
	}

	return subs;
    }


    private static void  checkOptionIndex ( String  option,
					    int     index,
					    int     argv_length )
    {
	if (index >= argv_length)
	{
	    System.err.println (JAVA_NAME + ": Bad option: " + option);
	    System.exit	       (1);
	}
    }


    private static int  getCheckInt (String  value_str)
    {
	try
	{
	    return Integer.parseInt (value_str);
	}
	catch (NumberFormatException  format_except)
	{
	    System.err.println (JAVA_NAME + ": " +
				format_except.getMessage () + ": " +
				value_str);
	    System.exit	       (1);
	}

	return -1;
    }


    private static void  printUsage ()
    {
	int  home_var_length	= HOME_VAR.length ();
	int  program_var_length = PROGRAM_VAR.length ();

	int  max_var_length = home_var_length;

	if (program_var_length > max_var_length)
	    max_var_length = program_var_length;

	System.err.print (
	    "Usage: " + JAVA_NAME + " [<option> [...]]\n" +
	    "\n" +
	    "Possible options:\n" +
	    "\n" +
	    "    {-help,-?}\n" +
	    "          Print this message and exit.\n" +
	    "\n" +
	    "    -debug\n" +
	    "          Print extra log debug information\n" +
	    "\n" +
	    "    -config_file {<config_file>," + STDIN_FILE + "}\n" +
	    "          Specify a configuration file to supply the " +
	    "program\n" +
	    "          parameters. They may be given in the format:\n" +
	    "\n" +
	    "              <name> = <value>\n" +
	    "\n" +
	    "          where <name> is one of the program options " +
	    "without\n" +
	    "          leading '-'. The <value> fields may be spread " +
	    "across multiple\n" +
	    "          lines. No line escape character is used. " +
	    "Lines with spaces\n" +
	    "          and comment text after '" + COMMENT_STR +
	    "' are ignored. When '" + STDIN_FILE + "' is\n" +
	    "          specified, Standard Input will be used. The " +
	    "following\n" +
	    "          variables are substituted by:\n" +
	    "\n" +
	    "              " + HOME_VAR +
	    getMultString (" ", max_var_length - home_var_length) +
	    "   The home directory of the user.\n" +
	    "              " + PROGRAM_VAR +
	    getMultString (" ", max_var_length - program_var_length) +
	    "   The base program name.\n" +
	    "\n" +
	    "    -home <home_dir>\n" +
	    "          Specify the home directory of the user. When " +
	    "undefined it\n" +
	    "          will be obtained from the system properties.\n" +
	    "\n" +
	    "    -program_dir <program_dir>\n" +
	    "          Specify the program directory where the " +
	    "commands to execute\n" +
	    "          are found. The default is: " + DFT_PROGRAM_DIR +
	    "\n" +
	    "\n" +
	    "    -program <program>\n" +
	    "          Specify the program used to substitute in " +
	    " the \"" + PROGRAM_VAR + "\n" +
	    "          variable. The default is: " + DFT_PROGRAM +
	    "\n" +
	    "\n" +
	    "    -log_file {<log_file>," + STDIN_FILE + "}\n" +
	    "          Specify a log file. The same variables as in " +
	    "the\n" +
	    "          \"-config_file\" option may be used.\n" +
	    "\n" +
	    "    -pid_file <pid_file>\n" +
	    "          This option will be ignored here.\n" +
	    "\n" +
	    "    -allow_host <host>\n" +
	    "          Allow only connections from the specified " +
	    "host. This option\n" +
	    "          may be multiple specified. Default " +
	    "connections from all hosts\n" +
	    "          are allowed.\n" +
	    "\n" +
	    "    -port <port>\n" +
	    "          Specify the port where this server listens. " +
	    "The default\n" +
	    "          is: " + DFT_PORT + ".\n"
	    );
    }


    private static String  getMultString ( String  base,
					   int	   nr )
    {
	String  mult = "";

	while (nr-- > 0)
	{
	    mult += base;
	}

	return mult;
    }


    boolean  getDebug ()
    {
	return debug;
    }


    String  getProgramDir ()
    {
	return program_dir;
    }


    String  getProgram ()
    {
	return program;
    }


    int  getPort ()
    {
	return port;
    }


    boolean  acceptAllHosts ()
    {
	return accept_all;
    }


    boolean  acceptHost (String  host)
    {
	return ( accept_all  ||  host_allow_hash.containsKey (host) );
    }


    boolean allowBaseCommand (String  base_com)
    {
	return base_com_allow_hash.containsKey (base_com);
    }


    boolean  useLogFile ()
    {
	return ( log_file != null  &&  !log_file.equals (STDIN_FILE) );
    }


    String  getLogFile ()
    {
	return log_file;
    }
}
