/* ------------------------------------------------------------------ */
/*		      D a t e F o r m a t . j a v a		      */
/*		      =============================		      */
/*								      */
/*   This file contains the source of the "DateFormat" class.	      */
/* ------------------------------------------------------------------ */


import java.util.*;


/**
 * This class specifies the format of a date and a time. It contains
 * comparison and conversion methods.
 * </P>
 *
 * <P>
 * The time field may also be substituted by an index (for instance the
 * number from the day or from the week within the year). In that
 * situation the time part is replaced bt midnight. An instance of the
 * <CODE>Hashtable</CODE> must me specified to link the date at the
 * index specified in the time field. Key is the date and time string
 * from the specified date at midnight.
 *
 *    @author  Hans Blom
 *
****************/
class  DateFormat
/***************/
{
    /**
     * The base time unit for seconds.
     */

    static final long  SECONDS = 1000;  // [ms]

    /**
     * The base time unit for minutes.
     */
     
    static final long  MINUTES = 60L * SECONDS;

    /**
     * The base time unit for hours.
     */
     
    static final long  HOURS   = 60L * MINUTES;

    /**
     * The base time unit for days.
     */

    static final long  DAYS    = 24L * HOURS;

    /**
     * The base time unit for weeks.
     */

    static final long  WEEKS   = 7L * DAYS;

    /*
     * Define some conversion constants, used during the conversion to
     * [ms].
     */

    private static final int  YEAR_OFFSET  = 1900;
    private static final int  MONTH_OFFSET =	1;

    /*
     * Define the separators between date and/or time values.
     */

    private static final char  DATE_TIME_SEPAR = '-';
    private static final char  DATE_SEPAR      = '/';
    private static final char  TIME_SEPAR      = ':';

    /*
     * Define the string representation of midnight.
     */

    private static final String  MIDNIGHT = "00" + TIME_SEPAR +
					    "00" + TIME_SEPAR +
					    "00";

    /*
     * Define the field indices in the date-time field and its field
     * length.
     */
    private static final int  DATE_IND		  = 0;
    private static final int  TIME_IND		  = 1;
    private static final int  NR_DATE_TIME_FIELDS = 2;

    /*
     * Define the field indices in the date field and its field length.
     */

    private static final int  DAY_IND	     = 0;
    private static final int  MONTH_IND	     = 1;
    private static final int  YEAR_IND	     = 2;
    private static final int  NR_DATE_FIELDS = 3;

    /*
     * Define the field indices in the time fields and its field length.
     */

    private static final int  HOUR_IND	     = 0;
    private static final int  MINUTE_IND     = 1;
    private static final int  SECOND_IND     = 2;
    private static final int  NR_TIME_FIELDS = 3;

    /*
     * Define the # digits used in the date and time fields.
     */

    private static final int  NR_DAY_DIG    = 2;
    private static final int  NR_MONTH_DIG  = 2;
    private static final int  NR_YEAR_DIG   = 4;
    private static final int  NR_HOUR_DIG   = 2;
    private static final int  NR_MINUTE_DIG = 2;
    private static final int  NR_SECOND_DIG = 2;

    /*
     * define the date and time field arrays. They are used to contain
     * the splitted strings.
     */

    private String []  date_time_fields;
    private String []  date_fields;
    private String []  time_fields;

    /*
     * Define the instance of class "Date".
     */

    Date  date;    // Contains a specified date.


    /**
     * Constructor. It initialises the field array to contain the
     * splitted fields. The date is initialised to <CODE>null</CODE>.
     *
    ************/
    DateFormat ()
    /***********/
    {
	/*
	 * Allocate the split arrays.
	 */

	date_time_fields = new String [NR_DATE_TIME_FIELDS];
	date_fields	 = new String [NR_DATE_FIELDS];
	time_fields	 = new String [NR_TIME_FIELDS];

	/*
	 * Initialise the date to unset.
	 */

	date = null;

    } // DateFormat ()


    /**
     * Set the date and time fields from an input string. In the place
     * of the time field also a date index may me specified. In that
     * case the <CODE>index_table</CODE> <CODE>Hashtable</CODE> should
     * be not <CODE>null</CODE>. The index will be placed in the
     * hashtable with as key the string of the specified date (without
     * the time part). In this way the date index is linked to the date.
     * </P>
     *
     * <P>
     * A time representation of the specified date-time string is
     * returned. When the time field is an date ID, it is replaced by
     * the time at midnight.
     *
     *    @param  date_time_str  The string which contains the date and
     *				 time fields.
     *
     *    @param  date_id_table	 When not <CODE>null</CODE> the time
     *				 field represents an week/day ID to be
     *				 stored in this hashtable.
     *
     *    @exception  Exception  An exception is throws after a bad
     *				 formatted date - time input string.
     *
    ********************************************************/
    String  set (String     date_time_str,
		 Hashtable  date_id_table)  throws  Exception
    /*******************************************************/
    {
	/*
	 * Split the date and time fields into date and time.
	 */

	StringTools.split (date_time_fields,
			   DATE_TIME_SEPAR,
			   date_time_str);

	/*
	 * Split the date fields.
	 */

	StringTools.split (date_fields,
			   DATE_SEPAR,
			   date_time_fields [DATE_IND]);

	if (date_id_table == null)    // A time field is specified.
	{
	    /*
	     * Split the time fields.
	     */

	    StringTools.split (time_fields,
			       TIME_SEPAR,
			       date_time_fields [TIME_IND]);

	    /*
	     * Obtain a new date with the specified date and time
	     * fields. Note that changing current time fields may
	     * generate errors in IE 5: the hour field is sometimes one
	     * hour to large during summer time.
	     */
	 
	    date = new Date (Integer.parseInt (date_fields [YEAR_IND]) -
			     YEAR_OFFSET,
			     Integer.parseInt (date_fields [MONTH_IND])
			     - MONTH_OFFSET,
			     Integer.parseInt (date_fields [DAY_IND]),
			     Integer.parseInt (time_fields [HOUR_IND]),
			     Integer.parseInt (time_fields [
						   MINUTE_IND]),
			     Integer.parseInt (time_fields [
						   SECOND_IND]));
	}
	else    // A date index is specified.
	{
	    /*
	     * Set the date at midnight.
	     */
	 
	    date = new Date (Integer.parseInt (date_fields [YEAR_IND]) -
			     YEAR_OFFSET,
			     Integer.parseInt (date_fields [MONTH_IND])
			     - MONTH_OFFSET,
			     Integer.parseInt (date_fields [DAY_IND]));

	    /*
	     * Link the index to the date (without time fieds), using
	     * the hashtable.
	     */

	    date_id_table.put (date_time_fields [DATE_IND],
			       date_time_fields [TIME_IND]);

	    /*
	     * Set the date-time string to represents the time at
	     * midnight.
	     */

	    date_time_str = date_time_fields [DATE_IND] +
			    DATE_TIME_SEPAR + MIDNIGHT;

	} // if (index_table == null)

	/*
	date.setYear    (Integer.parseInt (date_fields [YEAR_IND]) -
			 YEAR_OFFSET);
	date.setMonth   (Integer.parseInt (date_fields [MONTH_IND]) -
			 MONTH_OFFSET);
	date.setDate    (Integer.parseInt (date_fields [DAY_IND]));
	date.setHours   (Integer.parseInt (time_fields [HOUR_IND]));
	date.setMinutes (Integer.parseInt (time_fields [MINUTE_IND]));
	date.setSeconds (Integer.parseInt (time_fields [SECOND_IND]));
	*/

	/*
	 * Return the date-time representation of the current date -
	 * time/date ID.
	 */

	return date_time_str;
	
    } // set ()


    /**
     * Return the date (= day) part of a date&nbsp;- time class
     * instance.
     *
     *    @return  The date part.
     *
    *****************/
    String  getDate ()
    /****************/
    {
	StringBuffer  date_sb = new StringBuffer ();

	appendDate (date_sb);

	return date_sb.toString ();
    }


    /**
     * Return the time part of a date&nbsp;- time class instance.
     *
     *    @return  The time part.
     *
    *****************/
    String  getTime ()
    /****************/
    {
	StringBuffer  time_sb = new StringBuffer ();

	appendTime (time_sb);

	return time_sb.toString ();
    }


    /**
     * Convert a date&nbsp;- time field to a <CODE>String</CODE>
     * instance.  It is in the same form as the input string from which
     * this class instance was constructed by the method
     * <CODE>set</CODE>.
     *
     *    @return  The date&nbsp;- time field as a string.
     *
     *    @see  DateFormat#set
     *
    *************************/
    public String  toString ()
    /************************/
    {
	StringBuffer  format_sb = new StringBuffer ();

	appendDate	 (format_sb);
	format_sb.append (DATE_TIME_SEPAR);
	appendTime	 (format_sb);

	return format_sb.toString ();
    }


    /**
     * Append a date field from this class instance to an instance of a
     * <CODE>StringBuffer</CODE>. It is in the same form as the date
     * field in the input string.
     *
     *    @param  date_sb   The stringbuffer to append the date field
     *			    to.
     *
    ***********************************************/
    private void  appendDate (StringBuffer  date_sb)
    /**********************************************/
    {
	appendField    (date_sb,
			date.getDate (),
			NR_DAY_DIG);
	date_sb.append (DATE_SEPAR);
	appendField    (date_sb,
			date.getMonth () + MONTH_OFFSET,
			NR_MONTH_DIG);
	date_sb.append (DATE_SEPAR);
	appendField    (date_sb,
			date.getYear () + YEAR_OFFSET,
			NR_YEAR_DIG);
    }


    /**
     * Append a time field from this class instance to an instance of a
     * <CODE>StringBuffer</CODE>. It is in the same form as the time
     * field in the input string.
     *
     *    @param  date_sb   The stringbuffer to append the time field
     *			    to.
     *
    ***********************************************/
    private void  appendTime (StringBuffer  time_sb)
    /**********************************************/
    {
	appendField    (time_sb,
			date.getHours (),
			NR_HOUR_DIG);
	time_sb.append (TIME_SEPAR);
	appendField    (time_sb,
			date.getMinutes (),
			NR_MINUTE_DIG);
	time_sb.append (TIME_SEPAR);
	appendField    (time_sb,
			date.getSeconds (),
			NR_SECOND_DIG);

    } // appendTime ()


    /**
     * Get the date without the year field.
     *
     *    @return  The short date.
     *
    **********************/
    String  getShortDate ()
    /*********************/
    {
	StringBuffer  date_sb = new StringBuffer ();

	appendField    (date_sb,
			date.getDate (),
			NR_DAY_DIG);
	date_sb.append (DATE_SEPAR);
	appendField    (date_sb,
			date.getMonth () + MONTH_OFFSET,
			NR_MONTH_DIG);

	return date_sb.toString ();

    } // getShortDate ()


    /**
     * Get the time without the second field
     *
     *    @return  The short time.
     *
    **********************/
    String  getShortTime ()
    /*********************/
    {
	StringBuffer  time_sb = new StringBuffer ();

	appendField    (time_sb,
			date.getHours (),
			NR_HOUR_DIG);
	time_sb.append (TIME_SEPAR);
	appendField    (time_sb,
			date.getMinutes (),
			NR_MINUTE_DIG);

	return time_sb.toString ();

    } // getShortTime ()


    /**
     * Append a field to a stringbuffer with the required
     * #&nbsp;digits. Unsufficient digits are compensated by adding
     * zero digits before the appended field.
     *
     *    @param  date_sb     The stringbuffer to append the fiels to.
     *
     *    @param  field_val   The field value to append.
     *
     *    @param  nr_digits   The #&nbsp;digits which are required in
     *			      the string representation of the string.
     *
    ******************************************/
    void  appendField (StringBuffer  date_sb,
		       int	     field_val,
		       int	     nr_digits)
    /*****************************************/
    {
	/*
	 * Set the initial offset of the stringbuffer.
	 */

	int  field_offset = date_sb.length ();

	/*
	 * Append the field value (as a string) to the stringbuffer.
	 */

	date_sb.append (field_val);

	/*
	 * Calculate the # zeros to be prefixed before the field string.
	 */

	int  zero_offset = field_offset + nr_digits - date_sb.length ();

	/*
	 * Add the missing zero digits until the required field length
	 * is reached.
	 */

	while (zero_offset-- > 0)
	    date_sb.insert (field_offset, '0');
    
    } // appendField ()


    /**
     * Compare a date with this instance and return the result of the
     * comparison. The value is:
     *
     * <UL>
     *
     * <LI><B>&lt;&nbsp;0</B> when this date is older than the
     *	   specified date.</LI>
     *
     * <LI><B>&gt;&nbsp;0</B> when this date is more recent that the
     *	   specified date.</LI>
     *
     * <LI><B>zero</B> when both dates are equal.</LI>
     *
     * </UL>
     *    @param  cmp_date_format   The date format to compare with
     *				    this date.
     *
     *    @return  The result of the comparison.
     *
    *******************************************/
    int  compareTo (DateFormat  cmp_date_format)
    /******************************************/
    {
	Date  cmp_date = cmp_date_format.date;

	if (date.before (cmp_date))
	    return -1;
	else if (date.after (cmp_date))
	    return 1;
	else
	    return 0;

    } // compareTo ()


    /**
     * Set the time from a specified time.
     *
     *    @param  time   The time specified in [ms].
     *
    *****************************/
    void  setTimeInMS (long  time)
    /****************************/
    {
	if (date == null)	     // Date not yet defined: alloc. it.
	    date = new Date (time);
	else			     // Date was previously defined.
	    date.setTime (time);
    }


    /**
     * Return the contained time in [ms] in integral minutes.
     *
     *    @return   The time in [ms].
     *
    **************************/
    long  getTimeInMSSkipSec ()
    /*************************/
    {
	/*
	 * Define a new date from the current time, but without seconds.
	 */

	Date  min_date = new Date (date.getYear    (),
				   date.getMonth   (),
				   date.getDate	   (),
				   date.getHours   (),
				   date.getMinutes (),
				   0);
	/*
	 * Return the time in [ms].
	 */

	return min_date.getTime ();

    } // getTimeInMSSkipSec ()


    /**
     * Get the contained time in [ms], but rounded to the next hour.
     *
     *    @return   The time in [ms] rounded to the next hour.
     *
    ***************************/
    long  getNextHourTimeInMS ()
    /**************************/
    {
	/*
	 * Define a new date from the current date with minutes and
	 * seconds skipped.
	 */

	Date  prev_hour_date = new Date (date.getYear  (),
					 date.getMonth (),
					 date.getDate  (),
					 date.getHours (),
					 0,
					 0);
	/*
	 * Set a new date from the previous date, but with the addition
	 * of an hour.
	 */

	Date  next_hour_date = new Date (prev_hour_date.getTime () +
					 HOURS);

	/*
	 * Return this date in [ms].
	 */

	return next_hour_date.getTime ();

    } // getNextHourTimeInMS ()


    /**
     * Get the contained time in [ms], but rounded to the next minute.
     *
     *    @return   The time in [ms] rounded to the next minute.
     *
    *****************************/
    long  getNextMinuteTimeInMS ()
    /****************************/
    {
	/*
	 * Define a new date from the current date with seconds skipped.
	 */

	Date  prev_min_date = new Date (date.getYear	(),
					date.getMonth	(),
					date.getDate	(),
					date.getHours	(),
					date.getMinutes (),
					0);
	/*
	 * Set a new date from the previous date, but with the addition
	 * of a minute.
	 */

	Date  next_min_date = new Date (prev_min_date.getTime () +
					MINUTES);

	/*
	 * Return this date in [ms].
	 */

	return next_min_date.getTime ();

    } // getNextMinuteTimeInMS ()

} // class DateFormat
