/* ------------------------------------------------------------------ */
/*	      N e t D a t a D e s c r i p t o r . j a v a	      */
/*	      ===========================================	      */
/*								      */
/*   This class contains the source of the "NetDataDescriptor"	      */
/*   class.							      */
/* ------------------------------------------------------------------ */


import java.util.*;
import java.awt.*;


/**
 * This class is used to specify the characteristics of the various
 * net data types. The characteristics are described by the class
 * variables which can be set (get) by corresponding setter (getter)
 * methods.
 *
 *    @author  Hans Blom
 *
***********************/
class  NetDataDescriptor
/**********************/
{
    /*
     * Declare the class variables which are used to describe the
     * characteristics of the net data.
     */

    private String	   type_title;		// Title of the data tp.
    private int		   select_id;		// Selected file ID.
    private String	   date_axis_label;	// Label of the date ax.
    private int		   max_nr_domains;	// Max # domains to sel.
    private long	   max_domain_time;	// Use max time in #dom.
    private String []	   domain_items;	// Domain item labels.
    private long   []	   domain_times;	// Domain time values.
    private ControlChoice  domain_choice;       // Domain choice object.
    private long	   current_domain_time; // Current domain time.
    private Hashtable	   date_id_table;       // Hasht. with date IDs.
    private DateVarTable   date_vars;		// Date variable table.


    /**
     * Constructor.
     *
    *******************/
    NetDataDescriptor ()
    /******************/
    {
	/*
	 * Initialise the variables.
	 */

	type_title	    = null;
	select_id	    = -1;      // Default no select file ID.
	date_axis_label	    = null;
	max_nr_domains	    = 0;
	max_domain_time	    = 0L;
	domain_items	    = null;
	domain_times	    = null;
	domain_choice	    = null;
	current_domain_time = 0L;
	date_id_table	    = null;
	date_vars	    = null;
    }


    /**
     * Set the title of this data type.
     *
     *    @param  type_title   The specified data type title.
     *
    **************************************/
    void  setTypeTitle (String  type_title)
    /*************************************/
    {
	this.type_title = type_title;
    }


    /**
     * Get the title of this data type.
     *
     *    @return  The data type title.
     *
    **********************/
    String  getTypeTitle ()
    /*********************/
    {
	return type_title;
    }


    /**
     * Set the selected file ID. This parameter makes sense when there
     * are more data files the current type.
     *
     *    @param  select_id   The selected file ID to set.
     *
    *********************************/
    void  setSelectID (int  select_id)
    /********************************/
    {
	this.select_id = select_id;
    }


    /**
     * Set the selected file ID. This parameter makes sense when there
     * are more data files the current type.
     *
     *    @retur
     *
     *    @return  The selected file ID.
     *
    ******************/
    int  getSelectID ()
    /*****************/
    {
	return select_id;
    }


    /**
     * Set the label of the date (<VAR>X</VAR>)&nbsp;axis.
     *
     *    @param  date_axis_label   The specified label of the date
     *				    axis.
     *
    ***********************************************/
    void  setDateAxisLabel (String  date_axis_label)
    /**********************************************/
    {
	this.date_axis_label = date_axis_label;
    }


    /**
     * Get the label of the date (<VAR>X</VAR>)&nbsp;axis.
     *
     *    @return  The label of the date axis.
     *
    **************************/
    String  getDateAxisLabel ()
    /*************************/
    {
	return date_axis_label;
    }


    /**
     * Set the maximum #&nbsp;domain times. No more than this number of
     * the domains specified in <CODE>setDomains()</CODE> are used.
     * </P>
     *
     * <P>
     * To have effect this method must be called prior to a call to
     * <CODE>setDomains()</CODE>.
     *
     *    @param  max_nr_domains   The specified maximum #&nbsp;domains.
     *
     *    @see  NetDataDescriptor#setDomains
     *
     *    @see  NetDataDescriptor#setMaxDomainTime
     *
    ******************************************/
    void  setMaxNrDomains (int  max_nr_domains)
    /*****************************************/
    {
	this.max_nr_domains = max_nr_domains;
    }


    /**
     * Set the maximum domain time in [ms]. Only the domains specified
     * in <CODE>setDomains()</CODE> with a domain time less equal to the
     * maximum domain time are used. This method overrules the settings
     * by <CODE>setMaxNrDomains</CODE>.
     * </P>
     *
     * <P>
     * To have effect this method must be called prior to a call to
     * <CODE>setDomains()</CODE>.
     *
     *    @param  max_domain_time   The specified maximum domain time.
     *
     *    @see  NetDataDescriptor#setDomains
     *
     *    @see  NetDataDescriptor#setMaxNrDomains
     *
    *********************************************/
    void  setMaxDomainTime (long  max_domain_time)
    /********************************************/
    {
	this.max_domain_time = max_domain_time;
    }


    /**
     * Set the domain item labels and the domain times in [ms] to be
     * used in the domain choice of the plot window. Also a new domain
     * choice (instance of the <CODE>ControlChoice</CODE>) is created by
     * this method, using the specified default item and mouse enter
     * status message.
     * </P>
     *
     * <P>
     * Note that the actual #&nbsp;domains to be used can be limited by
     * calls to the methods <CODE>setMaxNrDomains()</CODE> or
     * <CODE>setMaxDomainTime()</CODE>. To be effective, these calls
     * must be done prior to a call to this method.
     * </P>
     *
     * <P>
     * It is assumed that the last domain item relates to the full
     * domain. This item is always added to the selector, so also when
     * the #&nbsp;domains are limited. When the #&nbsp;domains are
     * limited new arrays are allocated for the domains, because the
     * content must be changed. In the other cases the array pointers
     * are used without copying.
     * </P>
     *
     * <P>
     * The specified domains will be used in the
     * <CODE>ControlChoice</CODE> instance which is also created by this
     * mehod.
     *
     *    @param  domain_items       The specified domain item labels.
     *
     *    @param  domain_times       The specified domain times in [ms].
     *
     *    @param  default_item       The specified default item of the
     *				     new domain choice to be created by
     *				     this method.
     *
     *    @param  mouse_enter_mess   The status messages shown when the
     *				     mouse enters the domain choice.
     *
     *    @see  NetDataDescriptor#setMaxNrDomains
     *
     *    @see  NetDataDescriptor#setMaxDomainTime
     *
     *    @see  ControlChoice
     *
    *********************************************/
    void  setDomains (String []  domain_items,
		      long   []  domain_times,
		      String	 default_item,
		      String	 mouse_enter_mess)
    /********************************************/
    {
	/*
	 * Set the length of the specified domain and the index of the
	 * last specified domain (relates to the full domain) in help
	 * variables.
	 */

	int  src_domain_length  = domain_items.length;
	int  src_domain_max_ind = src_domain_length - 1;

	/*
	 * When specified: check for a maximum domain time.
	 */
	 
	if (max_domain_time > 0)
	{
	    max_nr_domains = 0;    // Overrule maximum # domains.

	    /*
	     * Set the maximum # domains as the last number with a time
	     * smaller than the maximum time, or set it to the maximum
	     * index of the specified domains.
	     */

	    while ( max_nr_domains < src_domain_max_ind  &&
		    domain_times [max_nr_domains] <= max_domain_time )
	    {
		max_nr_domains++;
	    }

	    max_nr_domains++;    // Add space for the full domain item.
	}

	/*
	 * Check if a maximum # domains is specified and if it is
	 * smaller than the specified # domains.
	 */

	if (max_nr_domains > 0  &&  max_nr_domains < src_domain_length)
	{
	    /*
	     * Limit the specified # domains. Allocate new arrays and
	     * copy the domains. The specified last domain is also
	     * copied to the last item from the limited domains.
	     */
	     
	    this.domain_items = new String [max_nr_domains];  // Create
	    this.domain_times = new long   [max_nr_domains];  // arrays.

	    /*
	     * Copy the items and times until the last limited item.
	     */
	     
	    int  max_dest_domain_ind = max_nr_domains - 1;

	    for (int i = 0;  i < max_dest_domain_ind;  i++)
	    {
		this.domain_items [i] = domain_items [i];
		this.domain_times [i] = domain_times [i];
	    }

	    /*
	     * Copy the last specified item (time) to the last limited
	     * item (time).
	     */

	    this.domain_items [max_dest_domain_ind] =
		domain_items [src_domain_max_ind];
	    this.domain_times [max_dest_domain_ind] =
		domain_times [src_domain_max_ind];
	}
	else    // Do not limit the domains: just set the pointers.
	{
	    this.domain_items = domain_items;
	    this.domain_times = domain_times;
	}

	/*
	 * Create the new domain choice class instance, connected with
	 * the specified domain and mouse enter status message.
	 */

	domain_choice = new ControlChoice (domain_items,
					   default_item,
					   mouse_enter_mess);

    } // setDomains ()


    /**
     * Get the domain item label at the specified index.
     *
     *    @param  index   The index from where the item must be
     *			  obtained.
     *
     *    @return  The domain item.
     *
     *    @see  NetDataDescriptor#getDomainTime
     *
    *********************************/
    String  getDomainItem (int  index)
    /********************************/
    {
	return domain_items [index];
    }


    /**
     * Get the domain time in [ms] at the specified index.
     *
     *    @param  index   The index from where the time must be
     *			  obtained.
     *
     *    @return  The domain time in [ms].
     *
     *    @see  NetDataDescriptor#getDomainItem
     *
    *******************************/
    long  getDomainTime (int  index)
    /******************************/
    {
	return domain_times [index];
    }


    /**
     * Get the <CODE>ControlChoice</CODE> object which is used in the
     * plot window to select the plot domains.
     *
     *    @return  The domain choice.
     *
    ********************************/
    ControlChoice  getDomainChoice ()
    /*******************************/
    {
	return domain_choice;
    }


    /**
     * Set the current domain time in [ms].
     *
     *    @param  current_domain_time   The specified current domain
     *					time.
     *
    *****************************************************/
    void  setCurrentDomainTime (long  current_domain_time)
    /****************************************************/
    {
	this.current_domain_time = current_domain_time;
    }


    /**
     * Get the current domain time in [ms].
     *
     *    @return  The current domain time.
     *
    ****************************/
    long  getCurrentDomainTime ()
    /***************************/
    {
	return current_domain_time;
    }


    /**
     * Set the hashtable which is used to store date ID's. If this
     * instance is <CODE>null</CODE>, no date ID's are used.
     *
     *    @param  date_id_table   The specified date ID table.
     *
    **********************************************/
    void  setDateIDTable (Hashtable  date_id_table)
    /*********************************************/
    {
	this.date_id_table = date_id_table;
    }


    /**
     * Get the hashtable which is used to store date ID's. If no such
     * table exits, <CODE>null</CODE> is returned.
     *
     *    @return  The date ID hashtable or <CODE>null</CODE> when it
     *		   does not exists.
     *
    ***************************/
    Hashtable  getDateIDTable ()
    /**************************/
    {
	return date_id_table;
    }


    /**
     * Set the table which is used to store some date dependent
     * variables.
     *
     *    @param  date_vars   The date variable table to set.
     *
    **********************************************/
    void  setDateVarTable (DateVarTable  date_vars)
    /*********************************************/
    {
	this.date_vars = date_vars;
    }


    /**
     * Get the table which is used to store some date dependent
     * variables. If no such table is found <CODE>null</CODE> is
     * returned.
     *
     *    @return  The date variable table.
     *
    *******************************/
    DateVarTable  getDateVarTable ()
    /******************************/
    {
	return date_vars;
    }
    
} // class NetDataDescriptor
