/* ------------------------------------------------------------------ */
/*		     P l o t M a r k e r . j a v a		      */
/*		     =============================		      */
/*								      */
/*   This file contains the source of the "PlotMarker" class in the   */
/*   "user.plot" package.					      */
/* ------------------------------------------------------------------ */


package user.plot;    // Define the plot package.

import java.awt.*;


/**
 * Define the plot markers used in the <CODE>PlotCanvas</CODE> class.
 *
 *    @see  PlotCanvas
 *
 *    @author  Hans Blom
 *
***********************/
public class  PlotMarker
/**********************/
{
    /**
     * marker ID when no markers are used.
     */
    public static final int  NONE	  =  0;
    /**
     * Diamond marker ID.
     */
    public static final int  DIAMOND	  =  1;
    /**
     * Square marker ID.
     */
    public static final int  SQUARE	  =  2;
    /**
     * Triangle marker ID.
     */
    public static final int  TRIANGLE	  =  3;
    /**
     * Octagon marker ID.
     */
    public static final int  OCTAGON	  =  4;
    /**
     * Diabolo marker ID.
     */
    public static final int  DIABOLO	  =  5;
    /**
     * X&nbsp;sign marker ID.
     */
    public static final int  X_SIGN	  =  6;
    /**
     * Plus sign marker ID.
     */
    public static final int  PLUS_SIGN	  =  7;
    /**
     * Y&nbsp;sign marker ID.
     */
    public static final int  Y_SIGN	  =  8;
    /**
     * Arrow marker ID.
     */
    public static final int  ARROW	  =  9;
    /**
     * Z&nbsp;sign marker ID.
     */
    public static final int  Z_SIGN	  = 10;
    /**
     * Rayed square marker ID.
     */
    public static final int  RAYED_SQUARE = 11;
    /**
     * Bushel marker ID.
     */
    public static final int  BUSHEL	  = 12;
    /**
     * Asteric marker ID.
     */
    public static final int  ASTERIC	  = 13;
    /**
     * The default marker ID.
     */
    public static final int  DEFAULT	  = DIAMOND;

    /**
     * Class instance constant when no marker is used.
     */

    public static final PlotMarker  NONE_MARKER = new PlotMarker (NONE);

    /**
     * Class instance constant of the default marker.
     */

    public static final PlotMarker  DEFAULT_MARKER = new PlotMarker (
							     DEFAULT);

    /**
     * Marker name when no marker is used.
     */
    public static final String  NONE_NAME	  = "none";
    /**
     * Diamond marker name. 
     */
    public static final String  DIAMOND_NAME	  = "diamond";
    /**
     * Square marker name. 
     */
    public static final String  SQUARE_NAME	  = "square";
    /**
     * Triangle marker name. 
     */
    public static final String  TRIANGLE_NAME	  = "triangle";
    /**
     * Octagon marker name. 
     */
    public static final String  OCTAGON_NAME	  = "octagon";
    /**
     * Diabolo marker name. 
     */
    public static final String  DIABOLO_NAME	  = "diabolo";
    /**
     * X&nbsp;sign marker name. 
     */
    public static final String  X_SIGN_NAME	  = "x_sign";
    /**
     * Plus sign marker name. 
     */
    public static final String  PLUS_SIGN_NAME	  = "plus_sign";
    /**
     * Y&nbsp;sign marker name. 
     */
    public static final String  Y_SIGN_NAME	  = "y_sign";
    /**
     * Arrow marker name. 
     */
    public static final String  ARROW_NAME	  = "arrow";
    /**
     * Z&nbsp;sign marker name. 
     */
    public static final String  Z_SIGN_NAME	  = "z_sign";
    /**
     * Rayed square marker name. 
     */
    public static final String  RAYED_SQUARE_NAME = "rayed_square";
    /**
     * Bushel marker name. 
     */
    public static final String  BUSHEL_NAME	  = "bushel";
    /**
     * Asteric marker name. 
     */
    public static final String  ASTERIC_NAME	  = "asteric";

    /**
     * Array with all markers.
     */

    public static final int []  ALL_MARKERS =
    {
	DIAMOND,
	SQUARE,
	TRIANGLE,
	OCTAGON,
	DIABOLO,
	X_SIGN,
	PLUS_SIGN,
	Y_SIGN,
	ARROW,
	Z_SIGN,
	RAYED_SQUARE,
	BUSHEL,
	ASTERIC
    };

    /**
     * Define the default size of the plot markers.
     */

    private static final int  DEFAULT_SIZE = 8;

    /**
     * Initialise the marker size constant. It is a static variable,
     * because it is valid for all markers.
     */

    private static int  size = DEFAULT_SIZE;

    /**
     * Declare the marker value ID class instance.
     */

    private int  value;


    /**
     * Constructor. The marker specified by its ID is created.
     *
     *    @param  value   The specified marker ID.
     *
    ******************************/
    public  PlotMarker (int  value)
    /*****************************/
    {
	this.value = value;
    }


    /**
     * A new marker is returned with the specified name. When no marker
     * exists with the specified name, a new instance of the default
     * marker is returned.
     *
     *    @param  value_name   The name of the specified marker name.
     *
     *    @return   A new instance of the specified marker.
     *
     *    @see  PlotMarker#getValueOf
     *
    *****************************************************/
    public static  PlotMarker valueOf (String  value_name)
    /****************************************************/
    {
	/*
	 * Parse the marker name.
	 */

	int  value = getValueOf (value_name);

	if (value == -1)	// make the default marker if name not
	    value = DEFAULT;	// understood

	/*
	 * Return a new instance of the parsed marker.
	 */

	return new PlotMarker (value);

    } // valueOf ()


    /**
     * The marker value is returned which corresponds with the specified
     * name. When no marker exists with the specified name,
     * <CODE>-1</CODE> is returned.
     *
     *    @param  value_name   The name of the specified marker.
     *
     *    @return   The value of the specified marker name.
     *
     *    @see  PlotMarker#valueOf
     *
    *************************************************/
    public static int  getValueOf (String  value_name)
    /************************************************/
    {
	/*
	 * Parse the marker name and return the marker value found.
	 */

	if (value_name.equals (NONE_NAME))
	    return NONE;
	else if (value_name.equals (DIAMOND_NAME))
	    return DIAMOND;
	else if (value_name.equals (SQUARE_NAME))
	    return SQUARE;
	else if (value_name.equals (TRIANGLE_NAME))
	    return TRIANGLE;
	else if (value_name.equals (OCTAGON_NAME))
	    return OCTAGON;
	else if (value_name.equals (DIABOLO_NAME))
	    return DIABOLO;
	else if (value_name.equals (X_SIGN_NAME))
	    return X_SIGN;
	else if (value_name.equals (PLUS_SIGN_NAME))
	    return PLUS_SIGN;
	else if (value_name.equals (Y_SIGN_NAME))
	    return Y_SIGN;
	else if (value_name.equals (ARROW_NAME))
	    return ARROW;
	else if (value_name.equals (Z_SIGN_NAME))
	    return Z_SIGN;
	else if (value_name.equals (RAYED_SQUARE_NAME))
	    return RAYED_SQUARE;
	else if (value_name.equals (BUSHEL_NAME))
	    return BUSHEL;
	else if (value_name.equals (ASTERIC_NAME))
	    return ASTERIC;
	else    // Marker name not found.
	    return -1;

    } // getValueOf ()


    /**
     * Set the marker size constant. It is used for all markers. The
     * best result gives powers of&nbsp;4.
     *
     *    @param  new_size   The new overall marker size.
     *
    ******************************************/
    public static void  setSize (int  new_size)
    /*****************************************/
    {
	size = new_size;
    }


    /**
     * Get the marker size constant.
     *
     *    @return   The overall marker size.
     *
    ****************************/
    public static int  getSize ()
    /***************************/
    {
	return size;
    }


    /**
     * Set a new marker ID.
     *
     *    @param  value   The specified marker ID.
     *
    ****************************/
    public void  set (int  value)
    /***************************/
    {
	this.value = value;
    }


    /**
     * Get the ID of this marker.
     *
     *    @return   The ID of this marker.
     *
    **********************/
    public int  getValue ()
    /*********************/
    {
	return value;
    }


    /**
     * Draw the marker. The specified position is the middlepoint of the
     * marker.
     *
     *    @param  x   The <I>X</I>-coordinate of the marker middlepoint.
     *
     *    @param  y   The <I>Y</I>-coordinate of the marker middlepoint.
     *
     *    @param  g   The used graphical context.
     *
    ******************************/
    public void  draw (int	 x,
		       int	 y,
		       Graphics  g)
    /*****************************/
    {
	draw (value, x, y, g);
    }


    /**
     * Draw the marker specified with its ID. The specified position is
     * the middlepoint of the marker.
     *
     *    @param  value   The specified marker ID value.
     *
     *    @param  x	  The <I>X</I>-coordinate of the marker
     *			  middlepoint.
     *
     *    @param  y	  The <I>Y</I>-coordinate of the marker
     *			  middlepoint.
     *
     *    @param  g	  The used graphical context.
     *
    *****************************************/
    public static void  draw (int	value,
			      int	x,
			      int	y,
			      Graphics  g)
    /****************************************/
    {
	switch (value)    // Draw the specified marker.
	{
	    case NONE :

		return;

	    case DIAMOND :

		drawDiamond (x, y, g);
		break;

	    case SQUARE :

		drawSquare (x, y, g);
		break;

	    case TRIANGLE :

		drawTriangle (x, y, g);
		break;

	    case OCTAGON :

		drawOctagon (x, y, g);
		break;


	    case DIABOLO :

		drawDiabolo (x, y, g);
		break;

	    case X_SIGN :

		drawXSign (x, y, g);
		break;

	    case PLUS_SIGN :

		drawPlusSign (x, y, g);
		break;

	    case Y_SIGN :

		drawYSign (x, y, g);
		break;

	    case ARROW :

		drawArrow (x, y, g);
		break;

	    /*
	    case BOWTIE :

		drawBowtie (x, y, g);
		break;
	    */

	    case Z_SIGN :

		drawZSign (x, y, g);
		break;

	    case RAYED_SQUARE :

		drawRayedSquare (x, y, g);
		break;

	    case BUSHEL :

		drawBushel (x, y, g);
		break;

	    case ASTERIC :

		drawAsteric (x, y, g);
		break;

	    default :   // draw default marker if name not understood

		draw (DEFAULT, x, y, g);
		break;
	}

    } // draw


    /**
     * Draw a diamond marker. The specified position is the middlepoint
     * of the marker.
     *
     *    @param  x   The <I>X</I>-coordinate of the marker middlepoint.
     *
     *    @param  y   The <I>Y</I>-coordinate of the marker middlepoint.
     *
     *    @param  g   The used graphical context.
     *
    ********************************************/
    public static void  drawDiamond (int       x,
				     int       y,
				     Graphics  g)
    /*******************************************/
    {
	g.drawLine (x, y, x, y - size/2);

	int []  xp = { x,	   x - size/2, x,	   x + size/2,
		       x          };
	int []  yp = { y + size/2, y,	       y - size/2, y,
		       y + size/2 };

	g.drawPolygon (xp, yp, xp.length);
    }


    /**
     * Draw a square marker. The specified position is the middlepoint
     * of the marker.
     *
     *    @param  x   The <I>X</I>-coordinate of the marker middlepoint.
     *
     *    @param  y   The <I>Y</I>-coordinate of the marker middlepoint.
     *
     *    @param  g   The used graphical context.
     *
    *******************************************/
    public static void  drawSquare (int       x,
				    int       y,
				    Graphics  g)
    /******************************************/
    {
	g.drawLine (x, y, x, y - size/2);
	g.drawRect (x - size/2, y - size/2, size, size);
    }


    /**
     * Draw a triangle marker. The specified position is the middlepoint
     * of the marker.
     *
     *    @param  x   The <I>X</I>-coordinate of the marker middlepoint.
     *
     *    @param  y   The <I>Y</I>-coordinate of the marker middlepoint.
     *
     *    @param  g   The used graphical context.
     *
    *********************************************/
    public static void  drawTriangle (int       x,
				      int       y,
				      Graphics  g)
    /********************************************/
    {
	g.drawLine (x, y, x, y - size/2);

	int []  xp = { x,	   x - size/2, x + size/2,
		       x          };
	int []  yp = { y - size/2, y + size/4, y + size/4,
		       y - size/2 };

	g.drawPolygon (xp, yp, xp.length);
    }


    /**
     * Draw an octagon marker. The specified position is the middlepoint
     * of the marker.
     *
     *    @param  x   The <I>X</I>-coordinate of the marker middlepoint.
     *
     *    @param  y   The <I>Y</I>-coordinate of the marker middlepoint.
     *
     *    @param  g   The used graphical context.
     *
    ********************************************/
    public static void  drawOctagon (int       x,
				     int       y,
				     Graphics  g)
    /*******************************************/
    {
	g.drawLine (x, y, x, y - size/2);

	int []  xp = { x - size/4, x - size/2, x - size/2, x - size/4,
		       x + size/4, x + size/2, x + size/2, x + size/4,
		       x - size/4 };
	int []  yp = { y - size/2, y - size/4, y + size/4, y + size/2,
		       y + size/2, y + size/4, y - size/4, y - size/2,
		       y - size/2 };

	g.drawPolygon (xp, yp, xp.length);
    }


    /**
     * Draw a diabolo marker. The specified position is the middlepoint
     * of the marker.
     *
     *    @param  x   The <I>X</I>-coordinate of the marker middlepoint.
     *
     *    @param  y   The <I>Y</I>-coordinate of the marker middlepoint.
     *
     *    @param  g   The used graphical context.
     *
    ********************************************/
    public static void  drawDiabolo (int       x,
				     int       y,
				     Graphics  g)
    /*******************************************/
    {
	g.drawLine (x + size/2, y - size/2, x - size/2, y + size/2);
	g.drawLine (x - size/2, y - size/2, x + size/2, y + size/2);
	g.drawLine (x + size/2, y - size/2, x - size/2, y - size/2);
	g.drawLine (x + size/2, y + size/2, x - size/2, y + size/2);
    }


    /**
     * Draw a X&nbsp;sign marker. The specified position is the
     * middlepoint of the marker.
     *
     *    @param  x   The <I>X</I>-coordinate of the marker middlepoint.
     *
     *    @param  y   The <I>Y</I>-coordinate of the marker middlepoint.
     *
     *    @param  g   The used graphical context.
     *
    ******************************************/
    public static void  drawXSign (int       x,
				   int       y,
				   Graphics  g)
    /*****************************************/
    {
	g.drawLine (x + size/2, y - size/2, x - size/2, y + size/2);
	g.drawLine (x - size/2, y - size/2, x + size/2, y + size/2);
    }


    /**
     * Draw a plus&nbsp;sign marker. The specified position is the
     * middlepoint of the marker.
     *
     *    @param  x   The <I>X</I>-coordinate of the marker middlepoint.
     *
     *    @param  y   The <I>Y</I>-coordinate of the marker middlepoint.
     *
     *    @param  g   The used graphical context.
     *
    *********************************************/
    public static void  drawPlusSign (int       x,
				      int       y,
				      Graphics  g)
    /********************************************/
    {
	g.drawLine (x,		y - size/2, x,		y + size/2);
	g.drawLine (x + size/2, y,	    x - size/2, y);
    }


    /**
     * Draw a y&nbsp;sign marker. The specified position is the
     * middlepoint of the marker.
     *
     *    @param  x   The <I>X</I>-coordinate of the marker middlepoint.
     *
     *    @param  y   The <I>Y</I>-coordinate of the marker middlepoint.
     *
     *    @param  g   The used graphical context.
     *
    ******************************************/
    public static void  drawYSign (int       x,
				   int       y,
				   Graphics  g)
    /*****************************************/
    {
	g.drawLine (x, y, x + size/2, y - size/2);
	g.drawLine (x, y, x - size/2, y - size/2);
	g.drawLine (x, y, x,	      y + size/2);
    }


    /**
     * Draw an arrow marker. The specified position is the middlepoint
     * of the marker.
     *
     *    @param  x   The <I>X</I>-coordinate of the marker middlepoint.
     *
     *    @param  y   The <I>Y</I>-coordinate of the marker middlepoint.
     *
     *    @param  g   The used graphical context.
     *
    ******************************************/
    public static void  drawArrow (int       x,
				   int       y,
				   Graphics  g)
    /*****************************************/
    {
	g.drawLine (x,		y - size/2, x,		y + size/2);
	g.drawLine (x,		y - size/2, x - size/2, y);
	g.drawLine (x,		y - size/2, x + size/2, y);
	g.drawLine (x + size/2, y,	    x - size/2, y);
    }

    /**
     * Draw a z&nbsp;sign marker. The specified position is the
     * middlepoint of the marker.
     *
     *    @param  x   The <I>X</I>-coordinate of the marker middlepoint.
     *
     *    @param  y   The <I>Y</I>-coordinate of the marker middlepoint.
     *
     *    @param  g   The used graphical context.
     *
    ******************************************/
    public static void  drawZSign (int       x,
				   int       y,
				   Graphics  g)
    /*****************************************/
    {
	g.drawLine (x + size/2, y - size/2, x - size/2, y + size/2);
	g.drawLine (x - size/4, y - size/4, x + size/4, y + size/4);
	g.drawLine (x + size/2, y - size/2, x - size/2, y - size/2);
	g.drawLine (x + size/2, y + size/2, x - size/2, y + size/2);
    }


    /**
     * Draw a rayed square marker. The specified position is the
     * middlepoint of the marker.
     *
     *    @param  x   The <I>X</I>-coordinate of the marker middlepoint.
     *
     *    @param  y   The <I>Y</I>-coordinate of the marker middlepoint.
     *
     *    @param  g   The used graphical context.
     *
    ************************************************/
    public static void  drawRayedSquare (int       x,
					 int       y,
					 Graphics  g)
    /***********************************************/
    {
	g.drawLine (x + size/2, y - size/2, x,		y);
	g.drawLine (x - size/2, y - size/2, x - size/4, y - size/4);
	g.drawLine (x - size/2, y + size/2, x - size/4, y + size/4);
	g.drawLine (x + size/2, y + size/2, x + size/4, y + size/4);
	g.drawRect (x - size/4, y - size/4, size/4, size/4);
    }


    /**
     * Draw a bushel marker. The specified position is the middlepoint
     * of the marker.
     *
     *    @param  x   The <I>X</I>-coordinate of the marker middlepoint.
     *
     *    @param  y   The <I>Y</I>-coordinate of the marker middlepoint.
     *
     *    @param  g   The used graphical context.
     *
    *******************************************/
    public static void  drawBushel (int       x,
				    int       y,
				    Graphics  g)
    /******************************************/
    {
	g.drawLine (x + size/2, y - size/2, x - size/2, y + size/2);
	g.drawLine (x - size/2, y - size/2, x + size/2, y + size/2);
	g.drawLine (x,		y + size/2, x,		y - size/2);
	g.drawLine (x - size/2, y,	    x + size/2, y);
    }


    /**
     * Draw an asteric marker. The specified position is the middlepoint
     * of the marker.
     *
     *    @param  x   The <I>X</I>-coordinate of the marker middlepoint.
     *
     *    @param  y   The <I>Y</I>-coordinate of the marker middlepoint.
     *
     *    @param  g   The used graphical context.
     *
    ********************************************/
    public static void  drawAsteric (int       x,
				     int       y,
				     Graphics  g)
    /*******************************************/
    {
	g.drawLine (x, y, x, y + size/2);

	int []  xp  = { x - size/2, x + size/2, x	   };
	int []  ypu = { y + size/4, y + size/4, y - size/2 };
	int []  ypb = { y - size/4, y - size/4, y + size/2 };

	g.drawPolygon (xp, ypu, xp.length);
	g.drawPolygon (xp, ypb, xp.length);
    }

} // class PlotMarker
